# Save name of script that's sourcing us. Used for printing messages.
SCRIPTNAME=$0

# Function to reset the debug log.
debug_init ()
{
    if [ -n "$CEA_GENACT_DEBUG_LOG" ]
    then
        rm -f $CEA_GENACT_DEBUG_LOG
        echo "CeA/ExtendIT Debug Log opened by $SCRIPTNAME on `hostname` (`uname`), `date`." > $CEA_GENACT_DEBUG_LOG
    fi
}

# Debug function. Reads debug message(s) from stdin when no parameters are given,
# otherwise prints whatever is on the command line to the debug log.
debug ()
{
    if [ -n "$CEA_GENACT_DEBUG_LOG" ]
    then
        if [ $# -ne 0 ]
        then
            echo "$SCRIPTNAME: [D] $*" >> $CEA_GENACT_DEBUG_LOG
        else
            while read line
            do
                echo "$SCRIPTNAME: [D] $line" >> $CEA_GENACT_DEBUG_LOG
            done
        fi
    fi
}

# Print resp. informational, warning, error and fatal messages to stderr.
info ()    { echo "$SCRIPTNAME: [I] $*" 1>&2; }
warning () { echo "$SCRIPTNAME: [W] $*" 1>&2; }
error ()   { echo "$SCRIPTNAME: [E] $*" 1>&2; }
fatal ()   { echo "$SCRIPTNAME: [F] $*" 1>&2; }

# Cleanup stub. Called by 'die'. Should be defined in each script using 'die' and
# remove all temporary stuff generated by that script.
cleanup () { debug "Dummy cleanup function called with exit code $1."; }

# Die function. Calls the cleanup function, optionally prints a message to stderr,
# and exits. The exit code is also passed to the cleanup function.
# Usage: die <exit-code> [exit-message]
die ()
{
    cleanup $1

    if [ -n "$2" ]
    then
        if [ $1 -eq 0 ]; then info "$2"; else fatal "$2"; fi
    fi

    exit $1
}

# Echo number of lines in a file to stdout.
linecount ()
{
    [ -n "$1" ] || die 999 "Internal Error: Function 'linecount' needs at least 1 argument."
    wc $1 | { read count rest; echo $count; }
}

# Echo a specific line or a range of lines from stdin to stdout.
# When linenum_end is omitted, only linenum_start will be echoed.
# Usage: echolines <linenum_start> [<linenum_end>] <infile >outfile
echolines ()
{
    FIRSTLINE=$1
    LASTLINE=$2

    [ -n "$FIRSTLINE" ] || die 999 "Internal Error: Function 'echolines' needs at least 1 argument."
    [ -n "$LASTLINE" ] || LASTLINE=$FIRSTLINE   # Handle case where just 1 argument is given.
    [ $FIRSTLINE -ge 1 ] || FIRSTLINE=1         # If firstline is not greater than or equal to 1, start at 1.
    [ $LASTLINE -ge 1 ] || return 0             # If lastline is not greater than or equal to 1, do nothing.
    [ $FIRSTLINE -le $LASTLINE ] || return 0    # If firstline is not lesser than or equal to lastline, do nothing.
    sed -n -e "${FIRSTLINE},${LASTLINE}p"
}

# Test access rights of specified file and exits if check not passed.
# Usage: testrights <filename> [<accessrigts>]
# access rights shold be specified as following:
#   "W" - wright
#   "R" - read
#   "X" - execute
#   Combinations, such as "RW" or "RWX" and other are also allowed.

testrights ()
{
  FILENAME=$1
  ACC=$2

  [ -n "$FILENAME" ] || die 999 "Internal Error: Function 'testrights' needs at least 1 argument."
  [ -n "$ACC" ] || return 0    # Do nothing if no rights specified.
  if [ "`echo $ACC | grep R`" ]
  then
    test -r $FILENAME || die 1 "File $FILENAME is not accesible for reading"
  fi
  if [ "`echo $ACC | grep W`" ]
  then
    test -w $FILENAME || die 1 "File $FILENAME is not accesible for writing"
  fi
  if [ "`echo $ACC | grep X`" ]
  then
    test -x $FILENAME || die 1 "File $FILENAME is not accesible for executing"
  fi

}
