#!/bin/sh
###########################################################BeginFile############
# Copyright:  (c) 2006 by Consul Risk Management B.V.  All Rights Reserved
#
# Name:
#   getoradblogs.sh
#
# Purpose:
#   Back-end actuator script that collects database records from certain 
#   Oracle-database table using Oracle's utility sqlplus. 
#
# History:
#   061127  ScienceSoft-SA  EE0609007:  Created.
#   061214  ScienceSoft-SA  EE0609007:  Cosmetic changes.
#   070117  ScienceSoft-SA  EE0609007:  Handle discspace shortage.
###########################################################EndFile##############
 
#######################################################
# Usage: our_exit <exit-code> [exit-message] 
#######################################################
our_exit()
{
  debug $2 
  finish_log_in_error
  exit $1
}
 
finish_log_in_error()
{
  rm -f $TEMPFILE1
  echo "CeA/ExtendIT Debug Log finished by $SCRIPTNAME on `hostname` (`uname`), `date`." >> $CEA_GENACT_DEBUG_LOG
}

test_sqlplus_code()
{
  ret_val=$?
  if [ ${ret_val} -ne 0 ]
  then
    debug "sqlplus returned(${ret_val}): $SQLPLUS_RETURN"
    our_exit 2 "Error occured logging into Oracle via Sqlplus !"
  fi
   
  retval=`echo "$SQLPLUS_RETURN" | grep -c "ORA-"`
  if [ $retval -eq 1 ]
  then
    # sqlplus return an ORA-xxx error
    our_exit 2 "Error, sqlplus returned: $SQLPLUS_RETURN"
  fi
}
##############################################################################3

# Source our function library. It contains generic functions. 
. $CEA_GENACT_FUNCLIB || exit 1
 
CEA_GENACT_DEBUG_LOG=../log/${PROPSFILE}.log
debug_init

[[ -n "${OPTION2}" ]] \
  || our_exit 1 "Option #2 'Oracle Instance' must be specified!"

[[ -n "${OPTION3}" ]] \
  || our_exit 1 "Option #3 'Oracle User Name' must be specified!"

[[ -n "${OPTION5}" ]] \
  || our_exit 1 "Option #5 'Oracle Home Path' must be specified!"

[[ -n "${PROPSFILE}" ]] \
  || our_exit 1 "Properties file must be specified!"

# Parameters needed for oracle access
ORA_USER=${OPTION3}
ORA_PWD=${OPTION4}
ORA_INSTANCE=${OPTION2}
ORACLE_HOME=${OPTION5}
export ORACLE_HOME 


######################################################################
# Check if ORACLE_HOME/bin/sqlplus is existing.                      #
######################################################################
debug "Check if ${ORACLE_HOME}/bin/sqlplus is existing..."
if [[ ! -s ${ORACLE_HOME}/bin/sqlplus ]]
then
  our_exit 2 "${ORACLE_HOME}/bin/sqlplus not found."
fi


######################################################################
# Check major Oracle database release number.                        #
######################################################################
debug "Check major Oracle database release number..."
SQLPLUS_RETURN=`${ORACLE_HOME}/bin/sqlplus -S ${ORA_USER}/${ORA_PWD}@${ORA_INSTANCE} << ! 2>>$CEA_GENACT_DEBUG_LOG
set feedback off; 
set echo off; 
set head off; 
set pagesize 0; 
select VALUE 
from NLS_DATABASE_PARAMETERS
where PARAMETER = 'NLS_RDBMS_VERSION';
exit;
!`
test_sqlplus_code

RDBMS_VERSION=`echo ${SQLPLUS_RETURN} | sed -e "s/\(.*\)\..*\..*\..*\..*/\1/"`
if [ ${RDBMS_VERSION} -gt 8 ]
then
  debug "Major Oracle database release number: ${RDBMS_VERSION} (OK)."
  if [ ${RDBMS_VERSION} -gt 10 ]
  then
    ${RDBMS_VERSION}='10'
  fi
else 
  our_exit 2 "Unsupported major Oracle database release number: ${RDBMS_VERSION} (NOK)."
fi


######################################################################
# Obtain timestamp value from properties file.                       #
######################################################################
if [[ -s ${PROPSFILE} ]]
then 
  . ./${PROPSFILE} || our_exit 3 "Error reading actuator state file ${PROPSFILE}"
fi


######################################################################
# Obtain specific table name, table description, timestamp column.   #
######################################################################
. ${OPTION1} || our_exit 3 "Can not read ${OPTION1}." 
eval "TABLE_DESC=\${TABLE_DESC${RDBMS_VERSION}}"
eval "TABLE_HEAD=\${TABLE_HEAD${RDBMS_VERSION}}"
eval "COL_TIME=\${COL_TIME${RDBMS_VERSION}}"
[ ! -z "${TABLE_NAME}" ] || our_exit 1 "Missing table name in ${OPTION1}."
[ ! -z "${TABLE_DESC}" ] || our_exit 1 "Missing table description in ${OPTION1}."
[ ! -z "${TABLE_HEAD}" ] || our_exit 1 "Missing table header in ${OPTION1}."
[ ! -z "${COL_TIME}" ] || our_exit 1 "Missing timestamp column in ${OPTION1}."
[ ! -z "${LINE_SIZE}" ] || LINE_SIZE='10000'

INITIAL_COLLECT=0
if [ -z "${LAST_TIME_BEFORE}" ]
then
  debug "Initial collect."
  INITIAL_COLLECT=1
  COND1=`echo "(LOCALTIMESTAMP-${COL_TIME}) < INTERVAL '2' DAY"`
else
  debug "Non-initial collect, start timestamp: '${LAST_TIME_BEFORE}'."
  COND1=`echo "${COL_TIME} >= to_date('${LAST_TIME_BEFORE}','DD-MON-YYYY HH24:MI:SS')"`
fi


######################################################################
# Obtain the greatest timestamp among all audit records.             #
######################################################################
debug "Obtain the greatest timestamp among all audit records..."
QUERY=`echo "select to_char(max(${COL_TIME}),'DD-MON-YYYY HH24:MI:SS') from ${TABLE_NAME}"`
SQLPLUS_RETURN=`${ORACLE_HOME}/bin/sqlplus -S ${ORA_USER}/${ORA_PWD}@${ORA_INSTANCE} << ! 2>>$CEA_GENACT_DEBUG_LOG
set feedback off; 
set echo off; 
set head off; 
set pagesize 0;
${QUERY};
exit;
!`
test_sqlplus_code
LAST_TIME_NOW=${SQLPLUS_RETURN}
debug "The greatest timestamp: ${LAST_TIME_NOW}'."


######################################################################
# Obtain actual audit records.                                       #
######################################################################
if [ -z "LAST_TIME_NOW" ]
then
  debug "The greatest timestamp is empty."
  if [ ${INITIAL_COLLECT} -eq 1 ]
  then
    LAST_TIME_NOW=${LAST_TIME_BEFORE}
  else
    DATE_NOW=$(date +"%d-%b-%Y")
    TIME_NOW=$(date +"%H:%M")
    LAST_TIME_NOW=$(../bin/goBackInTime $DATE_NOW $TIME_NOW 2880)
  fi
else
  debug "Obtain actual audit records..."
  COND2=`echo "${COL_TIME} < to_date('${LAST_TIME_NOW}','DD-MON-YYYY HH24:MI:SS')"`

  TEST_QUERY=`echo "select ${TABLE_DESC} from ${TABLE_NAME} where ${COND1} and ${COND2} and rownum=1 order by ${COL_TIME}"`
  SQLPLUS_RETURN=`${ORACLE_HOME}/bin/sqlplus -S ${ORA_USER}/${ORA_PWD}@${ORA_INSTANCE} << ! 2>>$CEA_GENACT_DEBUG_LOG
    set feedback off; 
    set echo off; 
    set head off; 
    set pagesize 0; 
    ${TEST_QUERY};
    exit;
  !`
  test_sqlplus_code

  TEST_QUERY2=`echo "select count(*) from ${TABLE_NAME} where ${COND1} and ${COND2}"`
  SQLPLUS_RETURN=`${ORACLE_HOME}/bin/sqlplus -S ${ORA_USER}/${ORA_PWD}@${ORA_INSTANCE} << ! 2>>$CEA_GENACT_DEBUG_LOG
    set feedback off; 
    set echo off; 
    set head off; 
    set pagesize 0; 
    ${TEST_QUERY2};
    exit;
  !`
  test_sqlplus_code
  
  RECORD_COUNT_EXP=${SQLPLUS_RETURN}
  debug "${RECORD_COUNT_EXP} actual audit records to be collected."
  RECORD_COUNT_EXP=`expr ${RECORD_COUNT_EXP} + 1`
  
  QUERY=`echo "select ${TABLE_DESC} from ${TABLE_NAME} where ${COND1} and ${COND2} order by ${COL_TIME}"`
  SQLPLUS_RETURN=`${ORACLE_HOME}/bin/sqlplus -S ${ORA_USER}/${ORA_PWD}@${ORA_INSTANCE} << ! >> /dev/null
  set newpage 0; 
  set feedback off; 
  set echo off; 
  set head off; 
  set pagesize 0; 
  set linesize ${LINE_SIZE};
  spool ${TEMPFILE1}0;
  select ${TABLE_HEAD} from dual;
  ${QUERY};
  exit;
  spool off;
  !`
  [ $? -eq 0 ] || our_exit 2 "Sqlplus returned ($?) an error."

  # remove the trailing spaces from the spool buffer   
  sed -e 's/ *$//' ${TEMPFILE1}0>${TEMPFILE1} || our_exit 3 "Internal error."
  rm ${TEMPFILE1}0 

  RECORD_COUNT_ACT=`wc -l ${TEMPFILE1} | sed -e 's/^ *\([0-9]\{1,\}\).*/\1/'`
  if [ ! ${RECORD_COUNT_ACT} -eq ${RECORD_COUNT_EXP} ]
  then
    rm ${TEMPFILE1}
    our_exit 3 "Audit record extraction was a fail, check disk space."
  fi
fi


######################################################################
# Store state information and finish log.                            #
######################################################################
echo "LAST_TIME_BEFORE='${LAST_TIME_NOW}'">${PROPSFILE}
debug "The collect process was a success (return code 0)" 
echo "CeA/ExtendIT Debug Log finished by $SCRIPTNAME on `hostname` (`uname`), `date`." >> $CEA_GENACT_DEBUG_LOG
